/*
** iplog_tcp.c - iplog TCP traffic logger.
** Copyright (C) 1999 Odin <odin@numb.org>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License, version 2,
** as published by the Free Software Foundation.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
*/

#include <config.h>

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <syslog.h>
#include <time.h>
#include <sys/types.h>
#include <pthread.h>

#include <iplog.h>
#include <iplog_options.h>
#include <iplog_filter.h>
#define DEF_ONLY
#include <iplog_scan.h>

#define TH_BOG		0xc0
#define SYNFLOOD	20

extern struct filter_data *filters[3];
extern u_long flags;

static time_t synlast;
static u_long last_bogus;
static u_char syncount;

pthread_mutex_t syn_lock = PTHREAD_MUTEX_INITIALIZER;
pthread_mutex_t bogus_lock = PTHREAD_MUTEX_INITIALIZER;

__inline__ bool tcp_res(void) {
	return ((flags & TCP_RES) && !(flags & NO_RESOLV) && syncount != SYNFLOOD);
}

int tcp_parser(struct ip *ip) {
	struct tcphdr *tcp = (struct tcphdr *) ((char *) ip + (ip->ip_hl << 2));
	time_t now = time(NULL);
	u_char lbuf[256];
	u_long len = ntohs(ip->ip_len) - (ip->ip_hl << 2);

	if (apply_filter(filters[FIL_TCP], ip, tcp->th_sport, tcp->th_dport))
		return (0);

	if (flags & SYN_FLOOD) {
		pthread_mutex_lock(&syn_lock);
		if (syncount > 0) {
			syncount -= (now - synlast) << 2;
			synlast = now;
		}
		pthread_mutex_unlock(&syn_lock);
	}
	
	if ((flags & NULL_SCAN) && !tcp->th_flags &&
		check_scan(SCAN_NULL, ip->ip_src.s_addr, len) != 0)
	{
		return (0);
	}

	if ((flags & XMAS_SCAN) && (tcp->th_flags & TH_FIN) &&
		(tcp->th_flags & TH_URG) && (tcp->th_flags & TH_PUSH) &&
		check_scan(SCAN_XMAS, ip->ip_src.s_addr, len) != 0)
	{
		return (0);
	}

	if ((flags & FIN_SCAN) && (tcp->th_flags & TH_FIN) &&
		!(tcp->th_flags & TH_ACK) &&
		check_scan(SCAN_FIN, ip->ip_src.s_addr, len) != 0)
	{
		return (0);
	}

	if ((flags & BOGUS) && (tcp->th_flags & TH_BOG)) {
		pthread_mutex_lock(&bogus_lock);
		if (last_bogus != ip->ip_src.s_addr) {
			last_bogus = ip->ip_src.s_addr;
			pthread_mutex_unlock(&bogus_lock);
			mysyslog(PRIORITY, "TCP: Bogus tcp flags set by %s",
				host_lookup(&ip->ip_src, tcp_res(),	lbuf, sizeof(lbuf)));
		} else
			pthread_mutex_unlock(&bogus_lock);

		return (0);
	}

	pthread_mutex_lock(&syn_lock);
	if (syncount != SYN_FLOOD)
		++syncount;
	pthread_mutex_unlock(&syn_lock);

	if ((tcp->th_flags & TH_SYN) && !(tcp->th_flags & TH_ACK)) {
		if ((flags & PORTSCAN) &&
			check_scan(SCAN_PORT, ip->ip_src.s_addr, len) != 0)
		{
				return (0);
		}

		if ((flags & GET_IDENT) && is_listening(tcp->th_dport)) {
			pthread_t pt;
			u_long buflen = ntohs(ip->ip_len);
			u_char *buf;

			buf = xmalloc(buflen);
			memcpy(buf, ip, buflen);
			pthread_create(&pt, NULL, get_ident_data, buf);
			pthread_detach(pt);

			return (0);
		} else {
			u_char sbuf[256];

			mysyslog(PRIORITY, "TCP: %s connection attempt from %s:%u",
				serv_lookup(tcp->th_dport, "tcp", sbuf, sizeof(sbuf)),
				host_lookup(&ip->ip_src, tcp_res(), lbuf, sizeof(lbuf)),
				ntohs(tcp->th_sport));
		}
	}

	return (0);
}

/* vim:ts=4:sw=8:tw=0 */
